% Loads command lines options
% Copyright (c) 2009 Ivan Meza (http://turing.iimas.unam.mx/~ivanvladimir)

:-use_module(library(lists),[memberchk/2,nth/3]).

% CLARGS as given by the option -a or the executable version
% DesOpts contains the possible options
% DesArgs contains the possible options
%
getOptsArgs(CLARGS,[DesOpts,DesArgs,DesCL],Opts,Args) :- 
   collect_options(false,DesOpts,SingleOpts),!,
   collect_options(true,DesOpts,ArgsOpts),!,
   parseCL(CLARGS,SingleOpts,ArgsOpts,DesArgs,Opts,Args),!.
   %collect_args_options(DesOpts,ArgsOpts).

% Parses de commandline
parseCL(Args,AS,AO,DA,O,A):-
    parseOCL(Args,AS,AO,O,RArgs),
    parseACL(RArgs,DA,A),!.

% Parse arguments
parseACL([],[],[]).
parseACL(Args,[[false,_]],[['args',Args]]):-
    check_noflags(Args).
parseACL([Arg|Args],[[true,_]],[['args',[Arg|Args]]]):-
    check_noflag(Arg),
    check_noflags(Args).
parseACL([Arg|Args],[[Des,true,_]|DAs],[[Des,Arg]|R]):-
    check_noflags(Args),
    parseACL(Args,DAs,R),!.
parseACL([],_,[]).

% Parse options
parseOCL([Arg1,Arg2|Args],AS,AO,[Os|O],RArgs):-
    check_flag(Arg1),
    (memberchk(Arg1,AS) ->
            Os=[Arg1],
            parseOCL([Arg2|Args],AS,AO,O,RArgs)
    | memberchk(Arg1,AO) ->
            check_noflag(Arg2),
            Os=[Arg1,Arg2],
            parseOCL(Args,AS,AO,O,RArgs)
    ). 
parseOCL([Arg|Args],AS,AO,Os,RArgs):-
    check_flag(Arg),
    (memberchk(Arg,AS) ->
        Os=[[Arg]|O],
        parseOCL(Args,AS,AO,O,RArgs)
    | 
        Os=[],
        RArgs=[Arg|Args]
    ).         
parseOCL(Args,_,_,[],Args).

check_noflags([]).
check_noflags([A|As]):-
    check_noflag(A), 
    check_noflags(As).

check_flag(A):-
    atom_chars(A,[45|_]).
check_noflag(A):-
    atom_chars(A,[ID|_]),
    ID =\= 45.


getOptArg(S,L,Vals,Def,Res):-
    (memberchk([S,Val],Vals) ->
        Res = Val
     |memberchk([L,Val2],Vals) ->
        Res = Val2
     |
        Res = Def
    ).

existsOptArg(S,L,Vals):-
    memberchk([S],Vals) | memberchk([L],Vals),!.




collect_options(_,[],[]):-!.
collect_options(F,[['',L,F,H]|RDes],[L|RArgs]):-
    collect_options(F,RDes,RArgs).
collect_options(F,[[S,'',F,H]|RDes],[S|RArgs]):-
    collect_options(F,RDes,RArgs).
collect_options(F,[[S,L,F,H]|RDes],[S,L|RArgs]):-
    collect_options(F,RDes,RArgs).
collect_options(F,[L|RDes],Args):-
    collect_options(F,RDes,Args).

%Test the command line

test_cl :- 
    test_cls([
          [['-v','--log','file.log','-t','file.t','file1','file2'],
           [[['-v','--verbose',false,'Prints verbose'],
             ['-l','--log',true,'Sets the logfile to print'],
             [],
             ['-t','--lll',true,'Sets the lll to print']],
            [['this_is_file',true,'First argument'],
             [true,'List of file']],
            ['This is a test.']
           ]],
          [['-v','--log','file.log','-t','file.t','file1','file2','file3','file4'],
           [[['-v','--verbose',false,'Prints verbose'],
             ['-l','--log',true,'Sets the logfile to print'],
             [],
             ['-t','--lll',true,'Sets the logfile to print']],
            [[true,'List of files']],
            ['This is a test']
           ]],
          [['-v','--log','-t','file.t','file1','file2','file3','file4'],
           [[['-v','--verbose',false,'Prints verbose'],
             ['-l','--log',true,'Sets the logfile to print'],
             [],
             ['-t','--lll',true,'Sets the logfile to print']],
            [[true,'List of files']],
            ['This is a test']
           ]]

           ]
        ).

print_help([Opts,Args,Lines]):-
    nl,
    print_help_lines(Lines),
    nl,
    print_help_args(Args),
    nl,
    print_help_opts(Opts).

print_help_lines([]).
print_help_lines([L|Ls]):-
    print(L),nl,
    print_help_lines(Ls).

print_help_args([]).
print_help_args([[]|R]):-
    nl,
    print_help_args(R).
print_help_args([[true,L]|R]):-
    print(L),nl,
    print_help_args(R).
print_help_args([[false,L]|R]):-
    print('['),print(L),print(']'),nl,
    print_help_args(R).
print_help_args([[I,true,L]|R]):-
    print(I),print('\t'),print(L),nl,
    print_help_args(R).
print_help_args([[I,false,L]|R]):-
    print('['),print(I),print(']'),print('\t'),print(L),nl,
    print_help_args(R).


print_help_opts([]).
print_help_opts([[]|R]):-
    nl,
    print_help_opts(R).
print_help_opts([['',L,D,H]|R]):-
    print('   '),print(L),print('\t'),print(H),nl,
    print_help_opts(R).
print_help_opts([[S,L,D,H]|R]):-
    print(S),print(','),print(L),print('\t'),print(H),nl,
    print_help_opts(R).


test_cls([]).
test_cls([[CL,D]|Rest]):-
    print('Command line args'),
    print(CL),nl,
    (getOptsArgs(CL,D,Opts,Args)->
        print('Options: '),
        print(Opts),nl,
        print('Arguments:'),
        print(Args),nl
     | print('Not passed'),nl,
       print_help(D)
    ),
    test_cls(Rest),!.

